//+------------------------------------------------------------------+
//|                                                  StringUtils.mqh |
//|                         Copyright (c) 2021-2025, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Concatenate all strings from array using glue symbol             |
//+------------------------------------------------------------------+
string StringCombine(const string &array[], const ushort glue)
{
   const int n = ArraySize(array);
   if(n == 0) return "";
   
   string result = array[0];
   
   for(int i = 1; i < n; ++i)
   {
      result += ShortToString(glue) + array[i];
   }
   return result;
}

//+------------------------------------------------------------------+
//| Array or subarray of strings concatenation                       |
//+------------------------------------------------------------------+
template<typename T>
string SubArrayCombine(const T &array[], const string glue = "",
   const uint start = 0, uint count = -1)
{
   const uint n = ArraySize(array);
   if(start >= n) return "";
   if(count == (uint)-1) count = n - start;
   
   string result = (string)array[start];
   
   for(uint i = 1; i < count && start + i < n; ++i)
   {
      result += glue + (string)array[start + i];
   }
   return result;
}

//+------------------------------------------------------------------+
//| Return a string with characters in reversed order                |
//+------------------------------------------------------------------+
string StringReverse(const string source)
{
   ushort chars[];
   return ArrayReverse(chars, 0, StringToShortArray(source, chars) - 1) ?
      ShortArrayToString(chars) : NULL;
}

//+------------------------------------------------------------------+
//| Return true/false if the first string ends with the second       |
//+------------------------------------------------------------------+
bool StringEndsWith(const string text, const string suffix)
{
  const int n = StringLen(text) - StringLen(suffix);
  return n >= 0 && StringFind(text, suffix, n) == n;
}

//+------------------------------------------------------------------+
//| Return two strings merged into one with a glue                   |
//+------------------------------------------------------------------+
string StringsCombine(const string prefix, const string glue, const string value)
{
   return prefix + (StringLen(prefix) && StringLen(value) ? glue : "") + value;
}

//+------------------------------------------------------------------+
//| Return updated string with uppercased 1-st char                  |
//+------------------------------------------------------------------+
string StringCapitalize(const string name)
{
   string firstLetter = StringSubstr(name, 0, 1);
   StringToUpper(firstLetter);
   return firstLetter + StringSubstr(name, 1);
}

//+------------------------------------------------------------------+
//| Replace a substring N times at most                              |
//+------------------------------------------------------------------+
int StringReplaceN(string &str, const string find, const string replacement, const int times)
{
   const int n = StringLen(find);
   const int m = StringLen(replacement);
   if(!n) return 0;
   if(times < 1) return 0;
   int count = 0;
   int start = 0;
   do
   {
      int p = StringFind(str, find, start);
      if(p > -1)
      {
         str = StringSubstr(str, 0, p) + replacement + StringSubstr(str, p + n);
         count++;
      }
      else break;
      start = p + m;
   } while(count < times);
   return count;
}

//+------------------------------------------------------------------+
